/**
 * @license
 * Copyright (c) 2016 The Polymer Project Authors. All rights reserved.
 * This code may only be used under the BSD style license found at http://polymer.github.io/LICENSE.txt
 * The complete set of authors may be found at http://polymer.github.io/AUTHORS.txt
 * The complete set of contributors may be found at http://polymer.github.io/CONTRIBUTORS.txt
 * Code distributed by Google as part of the polymer project is also
 * subject to an additional IP rights grant found at http://polymer.github.io/PATENTS.txt
 */
"use strict";
const fs = require('fs');
const path = require('path');
const logging = require('plylog');
// non-ES compatible modules
const swPrecache = require('sw-precache');
const Module = require('module');
let logger = logging.getLogger('cli.build.sw-precache');
function writeSWPrecache(destinationPath, config) {
    config.logger = logger.debug;
    return swPrecache.write(destinationPath, config);
}
function generateServiceWorker(options) {
    logger.debug(`generateServiceWorker() options:`, options);
    let swConfig = options.swConfig || {};
    // strip root prefix, so buildRoot prefix can be added safely
    let scriptsAndImports = options.deps;
    if (options.scriptAndStyleDeps) {
        scriptsAndImports = scriptsAndImports.concat(options.scriptAndStyleDeps);
    }
    let deps = scriptsAndImports.map((p) => {
        if (p.startsWith(options.root)) {
            return p.substring(options.root.length);
        }
        return p;
    });
    let mainHtml = options.entrypoint.substring(options.root.length);
    let precacheFiles = new Set(swConfig.staticFileGlobs);
    deps.forEach((p) => precacheFiles.add(p));
    precacheFiles.add(mainHtml);
    let precacheList = Array.from(precacheFiles);
    precacheList = precacheList.map((p) => path.join(options.buildRoot, p));
    // swPrecache will determine the right urls by stripping buildRoot
    swConfig.stripPrefix = options.buildRoot;
    // static files will be pre-cached
    swConfig.staticFileGlobs = precacheList;
    logger.debug(`writing service worker to ${options.serviceWorkerPath}`, swConfig);
    return writeSWPrecache(options.serviceWorkerPath, swConfig);
}
exports.generateServiceWorker = generateServiceWorker;
function parsePreCacheConfig(configFile) {
    return new Promise((resolve, reject) => {
        fs.stat(configFile, (err) => {
            let config;
            // only log if the config file exists at all
            if (!err) {
                try {
                    config = require(configFile);
                }
                catch (e) {
                    logger.warn(`${configFile} file was found but could not be loaded`, { err: err });
                }
            }
            resolve(config);
        });
    });
}
exports.parsePreCacheConfig = parsePreCacheConfig;