/**
 * @license
 * Copyright (c) 2016 The Polymer Project Authors. All rights reserved.
 * This code may only be used under the BSD style license found at http://polymer.github.io/LICENSE.txt
 * The complete set of authors may be found at http://polymer.github.io/AUTHORS.txt
 * The complete set of contributors may be found at http://polymer.github.io/CONTRIBUTORS.txt
 * Code distributed by Google as part of the polymer project is also
 * subject to an additional IP rights grant found at http://polymer.github.io/PATENTS.txt
 */
"use strict";
const fs = require('fs');
const path = require('path');
class ProjectConfig {
    constructor(configFile, options) {
        this._init(configFile, options);
    }
    static fromConfigFile(filepath) {
        try {
            let configContent = fs.readFileSync(filepath, 'utf-8');
            return JSON.parse(configContent);
        }
        catch (error) {
            // don't log if error is just "file not found"
            if (error.code !== 'ENOENT') {
                console.log('Could not load config file');
                console.error(error);
            }
        }
    }
    _init(configFile, options) {
        options = options || {};
        if (configFile) {
            // config file is default, options will override
            let fromFile = ProjectConfig.fromConfigFile(configFile);
            if (fromFile) {
                Object.assign(fromFile, options);
                options = fromFile;
            }
        }
        this.root = options.root || process.cwd();
        if (options.entrypoint) {
            this.entrypoint = path.resolve(this.root, options.entrypoint);
        }
        else {
            // fallback
            this.entrypoint = path.resolve(this.root, 'index.html');
            try {
                let bowerConfigContent = fs.readFileSync(path.resolve(this.root, 'bower.json'), 'utf-8');
                let bowerConfig = JSON.parse(bowerConfigContent);
                if (bowerConfig.main && typeof bowerConfig.main === 'string') {
                    this.entrypoint = path.resolve(this.root, bowerConfig.main);
                }
            }
            catch (_) { }
        }
        if (options.shell) {
            this.shell = path.resolve(this.root, options.shell);
        }
        this.fragments = [];
        // fragment comes from command-line-args `--fragment`
        let frag = options.fragment || options.fragments;
        if (frag) {
            this.fragments = frag.map((e) => path.resolve(this.root, e));
        }
        this.inputs = [];
        if (this.entrypoint)
            this.inputs.push(this.entrypoint);
        if (this.shell)
            this.inputs.push(this.shell);
        this.inputs = this.inputs.concat(this.fragments);
    }
}
exports.ProjectConfig = ProjectConfig;